# /usr/bin/env python3

import datetime
import RPi.GPIO as GPIO
import requests
import socket
import time
from apscheduler.schedulers.background import BackgroundScheduler
from apscheduler.triggers.interval import IntervalTrigger
from apscheduler.triggers.cron import CronTrigger
import configparser
import pytz

GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)
GPIO.setup(17, GPIO.IN, pull_up_down=GPIO.PUD_DOWN)
GPIO.setup(22, GPIO.IN, pull_up_down=GPIO.PUD_DOWN)

turnOnTime = 0
timeUpdated = False

scheduler = BackgroundScheduler()
timezone = pytz.timezone('Europe/Amsterdam')


def buttoncallback(channel):
    time.sleep(0.15)
    if GPIO.input(17) != GPIO.HIGH:
        pass
    else:
        requests.post(baseUrl + '/raspiswitch' + str(switchId) + '/override', data={'apikey': switchApiKey})


def toggleastral(channel):
    if GPIO.input(22) == GPIO.HIGH:
        requests.post(baseUrl + '/raspiswitch' + str(switchId) + '/toggleastral',
                      data={'command': 1, 'apikey': switchApiKey})
    else:
        requests.post(baseUrl + '/raspiswitch' + str(switchId) + '/toggleastral',
                      data={'command': 0, 'apikey': switchApiKey})


def syncastral():
    today = datetime.datetime.now()
    response = requests.get(baseUrl + '/raspiswitch' + str(switchId) + '/gettime',
                            params={'year': today.year, 'month': today.month, 'day': today.day, 'apikey': switchApiKey})
    global turnOnTime
    turnOnTime = datetime.datetime.strptime(response.json()['response'], '%a, %d %b %Y %H:%M:%S %Z')
    global timeUpdated
    timeUpdated = True


def usingastral():
    requests.get(baseUrl + '/raspiswitch' + str(switchId) + '/usingastral', params={'apikey': switchApiKey})


def isconnected():
    try:
        socket.create_connection(("www.google.com", 80))
        if centralLoggingEnabled:
            requests.post(baseUrl + '/raspapi/status',
                          data={'id': switchId, 'datetime': datetime.datetime.now(), 'key': 'conn_status', 'value': 1,
                                'apikey': serverApiKey})

    except OSError:
        if centralLoggingEnabled:
            requests.post(baseUrl + '/raspapi/status',
                          data={'id': switchId, 'datetime': datetime.datetime.now(), 'key': 'conn_status', 'value': 0,
                                'apikey': serverApiKey})
        else:
            print('connectionstatus ' + str(0))


# Turn off Switch
def turnoff():
    response = requests.post(baseUrl + '/raspiswitch' + str(switchId) + '/switchoff', params={'apikey': switchApiKey})
    if centralLoggingEnabled:
        requests.post(baseUrl + '/raspapi/cmd',
                      data={'id': switchId, 'datetime': datetime.datetime.now(),
                            'response': 'Turned ' + str(response.json()['currentstatus']), 'apikey': serverApiKey})
        requests.post(baseUrl + '/raspapi/status',
                      data={'id': switchId, 'datetime': datetime.datetime.now(), 'key': 'switch_status',
                            'value': int(response.json()['currentstatus']), 'apikey': serverApiKey})
    else:
        print(str(response.json()['response'] + ' ' + response.json()['currentstatus']))


# Turn on Switch
def turnon():
    response = requests.post(baseUrl + '/raspiswitch' + str(switchId) + '/switchon', data={'apikey': switchApiKey})
    if centralLoggingEnabled:
        requests.post(baseUrl + '/raspapi/cmd',
                      data={'id': switchId, 'datetime': datetime.datetime.now(),
                            'response': 'Turned ' + str(response.json()['currentstatus']), 'apikey': serverApiKey})
        requests.post(baseUrl + '/raspapi/status',
                      data={'id': switchId, 'datetime': datetime.datetime.now(), 'key': 'switch_status',
                            'value': int(response.json()['currentstatus']), 'apikey': serverApiKey})
    else:
        print(str(response.json()['response'] + ' ' + response.json()['currentstatus']))


# Initial timer setup
def setuptimers():
    response = requests.get(baseUrl + '/raspiswitch' + str(switchId) + '/usingastral', params={'apikey': switchApiKey})

    if bool(response.json()['currentstatus']) is True:
        global timeUpdated
        if timeUpdated is True:
            # Set up the scheduled time for the lights to be turned off
            if not scheduler.get_job('turnoff'):
                trigger = CronTrigger(hour=23, minute=55)
                scheduler.add_job(turnoff, trigger, id='turnoff')
                requests.post(baseUrl + '/raspapi/status',
                              data={'id': switchId, 'datetime': datetime.datetime.now(),
                                    'key': 'off_time', 'value': scheduler.get_job('turnoff').next_run_time,
                                    'apikey': serverApiKey})

            # Set up the scheduled time for the lights to be turned on
            global turnOnTime
            scheduler.add_job(turnon, 'date', run_date=turnOnTime, id='turnon')
            requests.post(baseUrl + '/raspapi/status',
                          data={'id': switchId, 'datetime': datetime.datetime.now(),
                                'key': 'on_time', 'value': turnOnTime, 'apikey': serverApiKey})
            timeUpdated = False


# Update timers from dashboard changes
def updatetimers():
    response = requests.post(baseUrl + '/raspapi/gettime',
                             params={'id': int(switchId), 'key': 'off_time', 'apikey': serverApiKey})
    time_processed = timezone.localize(datetime.datetime.combine(datetime.datetime.now().date(),
                                                                 datetime.datetime.strptime(response.json()['response'],
                                                                                            '%H:%M:%S').time()))
    # Reschedule turn off time
    if time_processed != scheduler.get_job('turnoff').next_run_time:
        print('rescheduling turn off')
        scheduler.reschedule_job(job_id='turnoff', trigger='date', run_date=time_processed)

    response = requests.post(baseUrl + '/raspapi/gettime',
                             params={'id': int(switchId), 'key': 'on_time', 'apikey': serverApiKey})
    time_processed = timezone.localize(datetime.datetime.combine(datetime.datetime.now().date(),
                                                                 datetime.datetime.strptime(response.json()['response'],
                                                                                            '%H:%M:%S').time()))
    # Reschedule turn on time
    if scheduler.get_job('turnon'):
        if time_processed != scheduler.get_job('turnon').next_run_time:
            print('rescheduling turn on')
            scheduler.reschedule_job(job_id='turnon', trigger='date', run_date=time_processed)


if __name__ == '__main__':
    config = configparser.ConfigParser()
    config.read('config.ini')
    baseUrl = config['setup']['baseurl']
    switchId = config['setup']['switchid']
    switchApiKey = config['setup']['apikey']
    serverApiKey = config['setup']['srvapikey']
    centralLoggingEnabled = config['setup']['storedb']

    # On startup
    # Check if lights are on (To make sure the LED is correct)
    requests.post(baseUrl + '/raspiswitch' + str(switchId) + '/lightstatus', data={'apikey': switchApiKey})

    # Sync Astral time (To set up timers)
    syncastral()
    time.sleep(1)
    # Check if the time will be used (To make sure server.py is correct)
    toggleastral(22)
    time.sleep(1)

    # Check every 30 seconds if wifi connection is present
    trigger = IntervalTrigger(seconds=30)
    scheduler.add_job(isconnected, trigger)

    # Check every 15 seconds if Astral is being used
    trigger = IntervalTrigger(seconds=15)
    scheduler.add_job(usingastral, trigger)

    # Update light on data everyday at 4:00
    trigger = CronTrigger(hour=4)
    scheduler.add_job(syncastral, trigger)

    # Overrideswitch button
    GPIO.add_event_detect(17, GPIO.RISING, callback=buttoncallback, bouncetime=800)

    # Astral Toggle button
    GPIO.add_event_detect(22, GPIO.BOTH, callback=toggleastral, bouncetime=800)

    # Turn the lights off when it is 23:55
    trigger = CronTrigger(hour=23, minute=55)
    scheduler.add_job(turnoff, trigger, id='turnoff')

    # Check every 10 minutes if time variables have been updated
    trigger = IntervalTrigger(minutes=10)
    scheduler.add_job(setuptimers, trigger)

    # Setup timer in case Astral will be activated later
    setuptimers()

    # Check if the timer values have been updated from the dashboard
    trigger = IntervalTrigger(minutes=5)
    scheduler.add_job(updatetimers, trigger)
    scheduler.start()

    requests.post(baseUrl + '/raspapi/status',
                  data={'id': switchId, 'datetime': datetime.datetime.now(),
                        'key': 'off_time', 'value': scheduler.get_job('turnoff').next_run_time})

    while True:
        time.sleep(60)
